@@ -1,5 +1,4 @@
-# This file is under MIT license. The license file can be obtained in the root
-# directory of this module.
+# This file is under MIT license. The license file can be obtained in the root directory of this module.
 
 
 """
@@ -28,9 +27,7 @@ class RootLayer:
         if(len(vector) != 4):
             raise ValueError('the length of the vector is not 4!')
         self._vector = vector
-        if(len(cid) != 16):
-            raise ValueError('the length of the CID is not 16!')
-        self._cid = cid
+        self.cid = cid
 
     def getBytes(self) -> list:
         '''Returns the Root layer as list with bytes'''
@@ -59,9 +56,10 @@ class RootLayer:
 
     @cid.setter
     def cid(self, cid: tuple):
-        if (len(cid) != 16 or not all(isinstance(x, int) for x in cid) or not all(0 <= x <= 255 for x in cid)):
+        if type(cid) is not tuple:
             raise TypeError(f'cid must be a 16 byte tuple! value was {cid}')
-        # derived classes must override this to init with appropriate length
+        if (len(cid) != 16 or not all((isinstance(x, int) and (0 <= x <= 255)) for x in cid)):
+            raise ValueError(f'cid must be a 16 byte tuple! value was {cid}')
         self._cid = cid
 
     def __eq__(self, other):
@@ -79,7 +77,7 @@ def int_to_bytes(integer_value: int) -> list:
     :return: the list with the high byte first
     """
     if not (isinstance(integer_value, int) and 0 <= integer_value <= 65535):
-        raise TypeError(f'integer_value to be packed must be unsigned short: [0-65535]! value was {integer_value}')
+        raise ValueError(f'integer_value to be packed must be unsigned short: [0-65535]! value was {integer_value}')
     return [(integer_value >> 8) & 0xFF, integer_value & 0xFF]
 
 
